######################################################################
#:CQCC::IOPtkResizeButton
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2001, 2008 All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# Copyright 2001 Rational Software Corporation. All Rights Reserved
# This software contains proprietary and confidential information of
# Rational and its suppliers.  Use, disclosure or reproduction is
# prohibited without the prior express written consent of Rational.

=head1 NAME 

CQCC::IOPtkResizeButton - PerlTk widget for resizeable column headers

=head1 CLASS DESCRIPTION

This class provides a specialized button for use in the column headers
used by IOPtkManip::ListboxGrid widget. It supports resize control
over the column that it heads as well as providing a normal button
callback.

=head1 METHODS

=cut

# Note: This was derived from resizeButton written by Shaun Wandler
#       and updated by Slaven Rezic which in turn was based on 
#       Columns.pm by Damion Wilson.
# 
# Bugs: 
# 1) ListboxGrid can stretch out the last column by itself which 
#    makes it look wider than it is and the resize margin can be 
#    misleading to find. 
# 2) Double click support for automatic resize was disabled
#    because the double click was also triggering a single click
#    event and causing column resort as well as column resize. 
######################################################################

######################################################################
#::Package declarations
######################################################################

require 5.006;
package CQCC::IOPtkResizeButton;

use base qw(Tk::Derived Tk::Button);

use Tk 800.022;
use Tk::HList;
require Tk::ItemStyle;

use strict;

sub Version { return '2.1' };

######################################################################
#::Class Globals 
######################################################################

my $edgeMargin = 10;   # Margin size on right edge that is resizeable
my $resizeFudge = 4;   # Adjustment to new width calculations

######################################################################
#::function ClassInit

=over 6

=item C<ClassInit()>

Method that initializes class based parameters and bindings. Invoked
automatically through Perl/TK calls.

=back 

=cut

######################################################################

Construct Tk::Widget 'IOPtkResizeButton';

sub ClassInit {
    my ($class,$mw) = @_;
    $class->SUPER::ClassInit($mw);
#    $mw->bind ($class, '<Double-Button-1>' ,'ButtonDoubleClick');
    $mw->bind ($class, '<ButtonRelease-1>' ,'ButtonRelease');
    $mw->bind ($class, '<ButtonPress-1>' ,'ButtonPress');
    $mw->bind ($class, '<Motion>' ,'ButtonOver');
    return $class;
}

######################################################################
#::function ClassTest

=over 6

=item C<ClassTest()>

This class function runs an internal class test producing a 
multi-column HList widget using that uses the resize buttons for
column headings. 

=back

=cut

######################################################################

sub ClassTest {
    my $top = Tk::MainWindow->new();
    my $cols = 5;
    my $listBox = 
       $top->Scrolled("HList", -header => 1, 
                      -columns => $cols,
                      )->pack(-expand =>1, -fill => 'both');

    my $headerstyle =
        $listBox->ItemStyle('window', -padx => 0, -pady => 0);

    my $scrolledHList = $listBox->Subwidget('scrolled');

    for (0 .. ($cols-1)) {
        my $widget = 
            $listBox->IOPtkResizeButton(-column => $_, 
                                        -text => "Column " . $_,
                                        -relief => 'flat', 
                                        -widget => \$scrolledHList,
                                        -command => 
                                           [ \&bCallback, $_ ]
                                        );
                                        
        $listBox->header('create', $_, 
                         -itemtype => 'window',
                         -widget => $widget, 
                         -style => $headerstyle);
    }

    MainLoop();
} 

######################################################################

sub bCallback {
    my (@args) = @_;
    print "bCallback (" . join(",",@args) . ")\n";
}

######################################################################
#::method Populate

=over 6

=item C<Populate()>

Method used by Perl/TK in building the widget and adding components
to the configuration specs.

=back

=cut

######################################################################

sub Populate {
    my ($self,$args) = @_;
    $self->SUPER::Populate($args);
    $self->ConfigSpecs(
        -widget => [['SELF', 'PASSIVE'], 'Widget', 'Widget', undef],
        -column => [['SELF', 'PASSIVE'], 'Column', 'Column', 0]
    );
}

######################################################################
#::method ButtonDoubleClick

=over 6

=item C<ButtonDoubleClick()>

Method called when the mouse button is double clicked over a button. 
This will cause the selected column to be reset to automatic resizing.

=back

=cut

######################################################################

sub ButtonDoubleClick{
    my ($self) = @_;

    if ($self->ButtonEdgeSelected()) {
        my $hlist = $self->cget('-widget');
        my $col = $self->cget('-column');
        $$hlist->columnWidth($col, "");
    }
}

######################################################################
#::method ButtonPress

=over 6

=item C<ButtonPress()>

Method called when the mouse button is pressed over the widget. 
If the mouse is in the resize margin at the right end of the widget
then the moveable column bar used to show the column edge is 
initialized, otherwise feedback is shown for a normal button hit. 

=back

=cut

######################################################################

sub ButtonPress{
    my ($self) = @_;

    $self->{RELIEF} = $self->cget('-relief');

    if ($self->ButtonEdgeSelected()) {
        $self->{EDGE_SELECTED} = 1;
        $self->{X} = $self->pointerx() - $self->rootx();
        ManageColumnBar($self);
    } else {
        $self->configure ('-relief' => 'sunken');
        $self->{X} = -1;
    }
}

######################################################################
#::method ButtonRelease

=over 6

=item C<ButtonRelease()>

Method called when the mouse button is released after being pressed.
If the button was in resize mode from the initial ButtonPress call
the moveable column bar is destroyed and the column width is adjusted 
accordingly. 

=back

=cut

######################################################################

sub ButtonRelease {
    my ($self) = @_;

    $self->{EDGE_SELECTED} = 0;
    $self->configure ('-relief' => $self->{RELIEF});

    if ($self->{columnBar}) { 
        $self->{columnBar}->destroy;
        undef $self->{columnBar};
    }

    if ($self->{X} >= 0) {
        my $newWidth = $self->pointerx() - $self->rootx() +
                         $self->{OFFSET} + $resizeFudge;
        # Don't permit resizing too small
        if ($newWidth < $edgeMargin) { 
            $newWidth = $edgeMargin;
        }
        my $hlist = $self->cget('-widget');
        my $col = $self->cget('-column');
        $$hlist->columnWidth($col, $newWidth);
        $self->GeometryRequest($newWidth, $self->reqheight());

    } elsif (! $self->ButtonEdgeSelected ()) {
        $self->Callback('-command');
    }

    $self->{X} = -1;
}

######################################################################
#::method ButtonEdgeSelected

=over 6

=item C<ButtonEdgeSelected()>

Method called to determine if the mouse is in the resize margin
on the right edge of the button. If it is when the button is
pressed the button will go into resize mode. 

=back

=cut

######################################################################

sub ButtonEdgeSelected{
   my ($self) = @_;
   return $self->rootx() + $self->width - $self->pointerx()
               <= $edgeMargin;
}

######################################################################
#::method ButtonOver

=over 6

=item C<ButtonOver()>

Method called while the mouse is over the button. If the button hasn't
been pressed yet, the cursor will show either the resize cursor
(<->) if the mouse is in the resize margin area or the normal left
pointer. If the button is in resize mode, the column bar is moved
to follow the mouse until the button is released. 

=back

=cut

######################################################################

sub ButtonOver{
    my($self) = @_;
    my ($cursor);
    if ($self->{EDGE_SELECTED} || 
        ($self->ButtonEdgeSelected() && 
         $self->cget('-relief') ne 'sunken')) {
        if ($self->{columnBar}) { 
            ManageColumnBar($self);
        }
        $cursor = 'sb_h_double_arrow' ;
    } else {
        $cursor = 'left_ptr';
    }
    $self->configure(-cursor => $cursor);
}

######################################################################
#::method ManageColumnBar

=over 6

=item C<ManageColumnBar()>

Method called to manage a temporary column bar shown to indicate
where the new column margin would be while the mouse is moving. 
A new column bar is created if needed; it will be destroyed when
the button is released.
      
=back

=cut

######################################################################

sub ManageColumnBar{
    my ($self) = @_;
    my $hlist  = $self->cget('-widget');
    my $height = $$hlist->height() - $self->height();
    my $x = $$hlist->pointerx() - $$hlist->rootx();

    if (! $self->{columnBar} ) { 
        $self->{OFFSET} = $self->rootx() + $self->width -
                          $$hlist->pointerx();
        $self->{columnBar} = 
            $$hlist->Frame( -background => 'white',
                            -relief => 'raised',
                            -borderwidth => 2,
                            -width => 2);
    }
    $self->{columnBar}->place(-x => $x + $self->{OFFSET}, 
                              -y => $self->height()+5,
                              -height => $height-5);
}

######################################################################
#::tail
######################################################################

1;

